﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace BookStore.Models
{
    /* These ViewModel classes contain all of the data needed to display the page for which they represent.
     * They do not contain any special logic other than some that may be in any constructors. */
    public class BookModel
    {
        public int id;
        public string title;
        public decimal price;
        public int pages;
        public string ISBN;
        public int inventory;
        public string description;
        public int numOfReviews = 0;
        public int averageRating;
        public List<Author> authors;

        public BookModel(AGET_BOOK book)
        {
            this.id = book.BOOKID;
            this.title = book.TITLE;
            this.price = (decimal) (from p in book.AGET_BOOKPRICE where p.ENDDATE == null select p.PRICE).FirstOrDefault();
            this.pages = book.PAGECOUNT;
            this.ISBN = book.ISBN;
            this.inventory = book.INVENTORYCOUNT;
            this.description = book.SYNOPSIS;
            this.numOfReviews = book.AGET_REVIEWS.Count();
            
            // Get the average rating if there is at least 1 review
            if(numOfReviews > 0)
                this.averageRating = (int)Math.Round((double)book.AGET_REVIEWS.Average(l => l.RATING));

            this.authors = (from a in book.AGET_AUTHOR
                             select new Author(a)).ToList();
        }
    }

    public class BookViewModel : BookModel
    {
        public bool reviewedByUser = false;
        public Publisher publisher;
        public List<Genre> genres;
        public List<Review> bookReviews;
        public List<BookModel> relatedBooks;

        public BookViewModel (AGET_BOOK book) : base(book)
        {
            this.relatedBooks = (from a in book.AGET_AUTHOR
                                 from b in a.AGET_BOOK
                                 where b.BOOKID != book.BOOKID
                                 group b by b.BOOKID into distinct
                                 select new BookModel(distinct.First())).Take(4).ToList();

            this.bookReviews = (from r in book.AGET_REVIEWS
                                orderby r.DATESUBMITTED
                                select new Review(r)).ToList();

            this.publisher = new Publisher(book.AGET_PUBLISHER);

            this.genres = (from g in book.AGET_GENRE
                           select new Genre(g)).ToList();

            // Find out if this book has been reviewed by the logged in user.
            User loggedInUser = HttpContext.Current.Session["UserSession"] as User;
            this.reviewedByUser = bookReviews.Where(l => l.username == loggedInUser.username).Count() >= 1;
        }
    }

    public class Author
    {
        public int id;
        public string name;

        public Author(AGET_AUTHOR author)
        {
            this.id = author.AUTHORID;
            this.name = author.NAME;
        }
    }

    public class Genre
    {
        public int id;
        public string name;

        public Genre(AGET_GENRE genre)
        {
            this.id = genre.GENREID;
            this.name = genre.NAME;
        }
    }

    public class Review
    {
        public string username;
        public string text;
        public short? rating;

        public Review(AGET_REVIEWS review)
        {
            this.username = review.AGET_USER.USERNAME;
            this.text = review.Text;
            this.rating = review.RATING;
        }
    }

    public class Publisher
    {
        public string name;
        public string information;

        public Publisher(AGET_PUBLISHER publisher)
        {
            this.name = publisher.NAME;
            this.information = publisher.INFORMATION;
        }
    }

    public class User
    {
        public int id;
        public string username;

        public User(AGET_USER user)
        {
            this.id = user.USERID;
            this.username = user.USERNAME;
        }
    }
}